"""AtCoder用のコマンド.

以下の構成からなる
- atcoder_helper.scripts: 実行のentry_pointを定義する
- atcoder_helper.services: まとまったひとつのユースケース群を定義する
- atcoder_helper.models: 主要なデータ構造を定義する
- atcoder_helper.repositories: 通信層を定義する

"""
